Term

Enum Term 

pub enum Term {
    Var {
        variable: V,
    },
    Const {
        constant: C,
    },
    App {
        function: F,
        terms: Vec<Term>,
    },
}
Represents a first-order term and consists of variables, constants and function applications.

Variants



Var

Is a variable term, wrapping a variable symbol.

Fields

variable: V


Const

Is a constant term, wrapping a constant symbol.

Fields

constant: C


App

Is a composite term, made by applying a function on a list of terms.

Fields

function: F
terms: Vec<Term>

Implementations

Source

impl Term

pub fn free_vars(&self) -> Vec<&V>

Returns a list of all free variable symbols in the term.

Note: In the list of free variables, each variable symbol appears only once even if it is present at multiple positions of the receiver term.

Example:


// `x_sym` and `y_sym` are variable symbols:
let x_sym = V::from("x");
let y_sym = V::from("y");

// `c_sym` is a constant symbol:
let c_sym = C::from("c");

// `x` and `y` are variable terms:
let x = Term::from(x_sym.clone());
let y = Term::from(y_sym.clone());

// `c` is a constant term:
let c = Term::from(c_sym.clone());

// `f` and `g` are function
let f = F::from("f");
let g = F::from("g");

// f(x, g(y, c, x)):
let t = f.app2(x.clone(), g.app3(y, c, x.clone()));

// comparing the two (unordered) lists:
assert_eq!(vec![&x_sym, &y_sym].iter().sorted(), t.free_vars().iter().sorted())
pub fn equals(self, term: Term) -> Formula

Returns an equation (formula) between the receiver and term.