pub struct C(/* private fields */);Expand description
Represents a constant symbol with a given name.
Note: Although it is possible to treat nullary functions as constants, we distinguish the two at a syntactic level.