pub struct F(/* private fields */);Expand description
Represents an uninterpreted function symbol with a given name.
Implementations
Sourceimpl F
impl F
Sourcepub fn app<T: FApp>(self, terms: Vec<T>) -> T
pub fn app<T: FApp>(self, terms: Vec<T>) -> T
Applies the receiver on a list of terms. The length of terms must be equal to
the (assumed) arity of the function.
Note: the definition of F does not impose any restrictions on the
arity of function symbols. The user is expected to assume the arity of the function.
Sourcepub fn app2<T: FApp>(self, first: T, second: T) -> T
pub fn app2<T: FApp>(self, first: T, second: T) -> T
Applies the (binary) receiver on two terms.
Sourcepub fn app3<T: FApp>(self, first: T, second: T, third: T) -> T
pub fn app3<T: FApp>(self, first: T, second: T, third: T) -> T
Applies the (ternary) receiver on three terms.