pub struct Pred(pub String);Expand description
Represents a predicate symbol with a given name.
Tuple Fields
0: StringImplementations
Sourceimpl Pred
impl Pred
Sourcepub fn app(self, terms: Vec<Term>) -> Formula
pub fn app(self, terms: Vec<Term>) -> Formula
Applies the receiver on a list of terms. The length of terms must be equal to
the (assumed) arity of the predicate.
Note: the definition of Pred does not impose any restrictions
on the arity of predicate symbols. The user is expected to assume the arity of the predicate.
Sourcepub fn app2(self, first: Term, second: Term) -> Formula
pub fn app2(self, first: Term, second: Term) -> Formula
Applies the (binary) receiver on two terms.
Sourcepub fn app3(self, first: Term, second: Term, third: Term) -> Formula
pub fn app3(self, first: Term, second: Term, third: Term) -> Formula
Applies the (ternary) receiver on three terms.