SkolemGenerator

Struct SkolemGenerator 

pub struct SkolemGenerator { /* private fields */ }
Generates Skolem names in an incremental fashion in the context of any transformation that involves Skolemization.

Note: To ensure all Skolem functions in a theory are unique, the same instance of SkolemGenerator must be used when transforming all formulae of the theory.

Implementations

Source

impl SkolemGenerator

pub fn new() -> Self

Creates a new SkolemGenerator with the default prefix "sk#".

pub fn next(&mut self) -> String

Returns the next Skolem name.