pub struct SkolemGenerator { /* private fields */ }Expand description
Generates Skolem names in an incremental fashion in the context of any transformation that involves Skolemization.
Note: To ensure all Skolem functions in a theory are unique, the same instance of
SkolemGenerator must be used when transforming all formulae of the theory.
Implementations